package com.yihu.hos.broker.controllers;

import com.yihu.hos.broker.services.camel.ESBCamelService;
import com.yihu.hos.broker.services.camel.SystemCamelContext;
import com.yihu.hos.web.framework.model.Result;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.camel.Route;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 流程管理
 * Created by lingfeng on 2016/8/4.
 */
@RestController
@RequestMapping("/esb")
@RefreshScope
public class ESBCamelController {
    @Autowired
    private ESBCamelService esbCamelService;
    private ExecutorService executorService = Executors.newWorkStealingPool();

    @RequestMapping(value = "/heartbeat", method = RequestMethod.GET)
    @ApiOperation(value = "测试服务器可以正常连接", notes = "测试服务器可以正常连接")
    public String heartbeat() {
        return "test";
    }

    @RequestMapping(value = "/serviceFlow", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation(value = "新增流程", produces = "application/json", notes = "当Arbiter通知一个的流程被创建时，该事件被触发")
    public Result onServiceFlowAdd(
            @ApiParam(name = "msg", value = "消息", required = true)
            @RequestBody() String msg) {
        executorService.execute(() -> esbCamelService.onServiceFlowAdd(msg));
        return Result.success("ok");
    }


    @RequestMapping(value = "/serviceFlow/add", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation(value = "修改流程", produces = "application/json", notes = "当Arbiter通知一个的流程被修改（Router，Process被修改或增加）时，该事件被触发")
    public Result onServiceFlowModifyAdd(
            @ApiParam(name = "msg", value = "消息", required = true)
            @RequestBody() String msg) {
        executorService.execute(() -> esbCamelService.onServiceFlowModifyAdd(msg));
        return Result.success("ok");
    }

    @RequestMapping(value = "/serviceFlow/reduce", produces = "application/json;charset=UTF-8", method = RequestMethod.PUT)
    @ApiOperation(value = "修改流程", produces = "application/json", notes = "当Arbiter通知一个的流程被修改（Router，Process被删除）时，该事件被触发")
    public Result onServiceFlowModifyReduce(
            @ApiParam(name = "msg", value = "消息", required = true)
            @RequestBody() String msg) {
        executorService.execute(() -> esbCamelService.onServiceFlowModifyReduce(msg));
        return Result.success("ok");
    }

    @RequestMapping(value = "/serviceFlow", produces = "application/json;charset=UTF-8", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除流程", produces = "application/json", notes = "当Arbiter通知一个的流程被删除时，该事件被触发")
    public Result onServiceFlowDelete(
            @ApiParam(name = "msg", value = "消息", required = true)
            @RequestBody() String msg) {
        executorService.execute(() -> esbCamelService.onServiceFlowDelete(msg));
        return Result.success("ok");
    }

    @RequestMapping(value = "/serviceFlow/start", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation(value = "启动服务流程", produces = "application/json", notes = "启动服务流程时，该事件被触发")
    public Result onServiceFlowStart(
            @ApiParam(name = "msg", value = "消息", required = true)
            @RequestBody() String msg) {
        executorService.execute(() -> esbCamelService.onServiceFlowStart(msg));
        return Result.success("ok");
    }

    @RequestMapping(value = "/serviceFlow/stop", produces = "application/json;charset=UTF-8", method = RequestMethod.POST)
    @ApiOperation(value = "停止服务流程", produces = "application/json", notes = "停止服务流程时，该事件被触发")
    public Result onServiceFlowStop(
            @ApiParam(name = "msg", value = "消息", required = true)
            @RequestBody() String msg) {
        executorService.execute(() -> esbCamelService.onServiceFlowStop(msg));
        return Result.success("ok");
    }

    @RequestMapping(value = "/serviceFlow/status", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation(value = "服务流程运行状态", produces = "application/json", notes = "查看服务流程的运行状态，运行中/停止")
    public String serviceFlowStatus(
            @ApiParam(name = "routeId", value = "routeId", required = true)
            @RequestParam String routeId) {
        return esbCamelService.getRouteStatus(routeId);
    }

    @RequestMapping(value = "/allFlow/status", produces = "application/json;charset=UTF-8", method = RequestMethod.GET)
    @ApiOperation(value = "所有服务流程运行状态", produces = "application/json", notes = "查看所有服务流程的运行状态，运行中/停止")
    public Map<String, String> allFlowStatus() {
        List<Route> routes = SystemCamelContext.getContext().getRoutes();
        Map<String, String> map = new HashMap<>();
        for (Route route : routes) {
            System.out.println("routId: " + route.getId() + " : " + SystemCamelContext.getContext().getRouteStatus(route.getId()));
            map.put(route.getId(), SystemCamelContext.getContext().getRouteStatus(route.getId()).name());
        }
        return map;
    }


}
