package com.yihu.hos.broker.services;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.yihu.hos.broker.configurations.ArbiterConfiguration;
import com.yihu.hos.core.http.HTTPResponse;
import com.yihu.hos.core.http.HttpClientKit;
import com.yihu.hos.core.net.IPChoiceUtils;
import org.apache.camel.util.HostUtils;
import org.apache.camel.util.InetAddressUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

/**
 * @created Airhead 2016/8/1.
 */
@Component("brokerServerService")
public class BrokerServerService {
    private final static Logger logger = LoggerFactory.getLogger(BrokerServerService.class);
    @Autowired
    private ArbiterConfiguration arbiterConfiguration;

    private String hostName;
    private String hostAddress;
    private int port;

    public void login() {
        try {
            hostName = InetAddressUtil.getLocalHostName();

            String host = System.getenv("host");
            if (host == null) {
                hostAddress = HostUtils.getLocalIp();
            } else {    //使用Docker方式时按固定的方式配置
                hostAddress = host;

            }
            port = 8099;    //目前先固定下来
            ObjectMapper objectMapper = new ObjectMapper();
            ObjectNode objectNode = objectMapper.createObjectNode();
            objectNode.put("tenant", arbiterConfiguration.getTenant());
            objectNode.put("hostName", hostName);
            objectNode.put("hostAddress", hostAddress);
            objectNode.put("port", port);
            objectNode.put("enable", true);
            String brokerServer = objectMapper.writeValueAsString(objectNode);

            Map<String, String> params = new HashMap<>();
            params.put("brokerServer", brokerServer);
            String url = arbiterConfiguration.getServer() + "/brokerServer/login";
            HTTPResponse response = HttpClientKit.post(url, params);
            if (response.getStatusCode() != 200) {
                logger.error("register broker server error, url:" + url + ",params:" + brokerServer + ",status code:" + response.getStatusCode());
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void heartbeat() {
        try {
            hostName = InetAddressUtil.getLocalHostName();

            String host = System.getenv("host");
            if (host == null) {
                hostAddress = IPChoiceUtils.getSourceIP();
            } else {    //使用Docker方式时按固定的方式配置
                hostAddress = host;

            }
            port = 8099;    //目前先固定下来
            ObjectMapper objectMapper = new ObjectMapper();
            ObjectNode objectNode = objectMapper.createObjectNode();
            objectNode.put("tenant", arbiterConfiguration.getTenant());
            objectNode.put("hostName", hostName);
            objectNode.put("hostAddress", hostAddress);
            objectNode.put("port", port);
            objectNode.put("enable", true);
            String brokerServer = objectMapper.writeValueAsString(objectNode);

            Map<String, String> params = new HashMap<>();
            params.put("brokerServer", brokerServer);
            String url = arbiterConfiguration.getServer() + "/brokerServer/heartbeat";
            HTTPResponse response = HttpClientKit.post(url, params);
            if (response.getStatusCode() != 200) {
                logger.error("heartbeat broker server error, url:" + url + ",params:" + brokerServer + ",status code:" + response.getStatusCode());
            }

        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * brokerServer下线时通知
     */
    public void logout() {
        try {
            ObjectMapper objectMapper = new ObjectMapper();
            ObjectNode objectNode = objectMapper.createObjectNode();
            objectNode.put("hostName", hostName);
            objectNode.put("hostAddress", hostAddress);
            objectNode.put("port", port);
            objectNode.put("enable", false);
            String brokerServer = objectMapper.writeValueAsString(objectNode);
            Map<String, String> params = new HashMap<>();
            params.put("brokerServer", brokerServer);

            String url = arbiterConfiguration.getServer() + "/brokerServer/logout";
            HTTPResponse response = HttpClientKit.delete(url, params);
            if (response.getStatusCode() != 200) {
                logger.error("logout broker server error, url:" + url + ",params:" + brokerServer + ",status code:" + response.getStatusCode());
            }

        } catch (IOException e) {
            e.printStackTrace();
        }

    }
}
