package com.yihu.hos.broker.services.camel;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.mongodb.MongoClient;
import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;
import com.mongodb.client.model.Filters;
import com.yihu.hos.core.constants.CoreConstant;
import com.yihu.hos.core.datatype.ClassFileUtil;
import com.yihu.hos.core.datatype.StringUtil;
import com.yihu.hos.web.framework.constant.ServiceFlowConstant;
import com.yihu.hos.web.framework.model.Result;
import com.yihu.hos.web.framework.model.bo.ServiceFlow;
import org.apache.camel.Route;
import org.apache.camel.ServiceStatus;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.bson.Document;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.stereotype.Service;

import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

/**
 * 对于Broker来说，Camel路由的加载只有这样的几个过程。
 * 文件相关
 * 1.Java， Class文件下载
 * 2.Java文件编译
 * 3.Class文件上传（有java编译生成的class的文件更新）
 * 4.Java，Class删除文件
 * Camel相关
 * 1.增加服务流程
 * 2.删除服务流程
 * 3.修改服务流程
 * 3.1增加Router，Processor
 * 3.2减少Router，Processor
 * 4.启动服务流程
 * 5.停止服务流程
 * <p>
 * Created by lingfeng on 2016/8/4.
 */
@RefreshScope
@Service
public class ESBCamelService {
    private static Logger logger = LogManager.getLogger(ESBCamelService.class);
    @Autowired
    private MongoClient mongoClient;

    private String serviceFlow = "serviceFlow";
    private String configuration = "runtime";
    @Autowired
    private ObjectMapper objectMapper;
    @Value("${hos.filesystem.url}")
    private String fsUrl;
    @Value("${spring.data.mongodb.gridFsDatabase}")
    private String dbName;

    public Result onServiceFlowAdd(String msg) {
        try {
            System.out.println("hos-broker 流程添加开始，onServiceFlowModifyAdd");
            ServiceFlowValid serviceFlowValid = new ServiceFlowValid(msg).invoke();
            if (serviceFlowValid.is()) return Result.error("必要的入参数据不正确，请检查！");

            ServiceFlow serviceFlow = serviceFlowValid.getServiceFlow();
            ArrayList<ServiceFlow.HandleFile> handleFiles = serviceFlowValid.getHandleFiles();
            if ("java".equals(serviceFlowValid.getServiceFlow().getFlowType())) {
                final boolean[] created = new boolean[1];
                //先生成和编译 processor
                handleFiles.stream().filter(handleFile -> ServiceFlowConstant.FLOW_TYPE_PROCESSOR.equals(handleFile.getUsage())).forEach(handleFile -> {
                    try {
                        if (this.generateClassFile(handleFile)){
                            created[0] = true;
                        } else {
                            created[0] = false;
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });

                if (!created[0]) {
                    System.out.println("onServiceFlowAdd ,生成processor文件失败，");
                    return Result.error("服务流程启动失败！");
                }

                handleFiles.stream().filter(handleFile -> ServiceFlowConstant.FLOW_TYPE_ROUTE.equals(handleFile.getUsage())).forEach(handleFile -> {
                    try {
                        if (this.generateClassFile(handleFile)) created[0] = true;
                        else created[0] = false;
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });
                if (!created[0]) {
                    System.out.println("onServiceFlowAdd ,生成route文件失败，");
                    return Result.error("服务流程启动失败！");
                }
            } else {
                for (ServiceFlow.HandleFile handleFile : handleFiles) {
                    boolean created = this.createClassFile(handleFile);
                    if (!created) {
                        return Result.error("服务流程启动失败！");
                    }
                }
            }

            logger.debug("add camel router" + serviceFlow.getRouteCode());
            this.addRouter(serviceFlow.getHandleFiles());

            logger.debug("start camel router," + serviceFlow.getRouteCode());
            SystemCamelContext.startRouter(serviceFlow.getRouteCode());

            logger.debug("start camel router success," + serviceFlow.getRouteCode());
            return Result.error("服务流程增加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            logger.error(e);
            return Result.error("服务流程增加失败！");
        }
    }

    /**
     * 当外界组件通知一个新的processor处理器被定义时，该事件被触发。
     */
    public Result onServiceFlowModifyAdd(String msg) {
        try {
            System.out.println("hos-broker 流程修改开始，onServiceFlowModifyAdd");
            ServiceFlowValid serviceFlowValid = new ServiceFlowValid(msg).invoke();
            if (serviceFlowValid.is()) return Result.error("必要的入参数据不正确，请检查！");
            List<Route> routes = SystemCamelContext.getContext().getRoutes();

            ServiceFlow serviceFlow = serviceFlowValid.getServiceFlow();
            SystemCamelContext.stopRouter(serviceFlow.getRouteCode());
            boolean b = SystemCamelContext.removeRouter(serviceFlow.getRouteCode());
            ArrayList<ServiceFlow.HandleFile> handleFiles = serviceFlowValid.getHandleFiles();

            if ("java".equals(serviceFlowValid.getServiceFlow().getFlowType())) {
                final boolean[] created = new boolean[1];
                //先生成和编译 processor
                handleFiles.stream().filter(handleFile -> ServiceFlowConstant.FLOW_TYPE_PROCESSOR.equals(handleFile.getUsage())).forEach(handleFile -> {
                    try {
                        if (this.generateClassFile(handleFile)) created[0] = true;
                        else created[0] = false;
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });

                if (!created[0]) {
                    return Result.error("服务流程启动失败！");
                }

                handleFiles.stream().filter(handleFile -> ServiceFlowConstant.FLOW_TYPE_ROUTE.equals(handleFile.getUsage())).forEach(handleFile -> {
                    try {
                        if (this.generateClassFile(handleFile)) created[0] = true;
                        else created[0] = false;
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });
                if (!created[0]) {
                    return Result.error("服务流程启动失败！");
                }
            } else {
                for (ServiceFlow.HandleFile handleFile : handleFiles) {
                    boolean created = this.createClassFile(handleFile);
                    if (!created) {
                        return Result.error("服务流程启动失败！");
                    }
                }
            }

            this.addRouter(serviceFlowValid.getHandleFiles());
            SystemCamelContext.startRouter(serviceFlow.getRouteCode());
            System.out.println("hos-broker 流程修改结束，onServiceFlowModifyAdd");
            return Result.error("服务流程变更增加成功！");
        } catch (Exception e) {
            e.printStackTrace();
//            logger.error(e);
            return Result.error("服务流程变更增加失败！");
        }
    }

    /**
     * 当外界组件通知一个已有的RouteDefine路由定义被删除时，该事件被触发。
     */
    public Result onServiceFlowModifyReduce(String msg) {
        try {
            ServiceFlowValid serviceFlowValid = new ServiceFlowValid(msg).invoke();
            if (serviceFlowValid.is()) return Result.error("必要的入参数据不正确，请检查！");

            ServiceFlow serviceFlow = serviceFlowValid.getServiceFlow();
            ServiceFlow.HandleFile handleFile = serviceFlowValid.getHandleFile();
            String routeCode = serviceFlow.getRouteCode();
            SystemCamelContext.getContext().stopRoute(routeCode);
            SystemCamelContext.getContext().removeRoute(routeCode);
            this.deleteClassFile(handleFile);
            this.addRouter(serviceFlowValid.getHandleFiles());
            SystemCamelContext.startRouter(serviceFlow.getRouteCode());

            return Result.success("服务流程变更减少成功！");
        } catch (Exception e) {
            return Result.error("服务流程变更减少失败！");
        }
    }

    public Result onServiceFlowDelete(String msg) {
        try {
            ServiceFlowValid serviceFlowValid = new ServiceFlowValid(msg).invoke();
            if (serviceFlowValid.is()) return Result.error("必要的入参数据不正确，请检查！");

            ServiceFlow serviceFlow = serviceFlowValid.getServiceFlow();
            String routeCode = serviceFlow.getRouteCode();
            SystemCamelContext.getContext().stopRoute(routeCode);
            boolean b = SystemCamelContext.getContext().removeRoute(routeCode);

            ArrayList<ServiceFlow.HandleFile> handleFiles = serviceFlowValid.getHandleFiles();
            if (handleFiles != null) {
                for (ServiceFlow.HandleFile handleFile : handleFiles) {
                    this.deleteClassFile(handleFile);
                    this.deleteServiceFlow(handleFile.getRouteCode());//删除mongo中serviceFlow信息
                }
            }
            return Result.success("服务流程删除成功！");
        } catch (Exception e) {
            return Result.error("服务流程删除失败！");
        }
    }

    /**
     * 启动路由时，该事件被触发。
     */
    public Result onServiceFlowStart(String msg) {
        try {
            ServiceFlowValid serviceFlowValid = new ServiceFlowValid(msg).invoke();
            if (serviceFlowValid.is()) return Result.error("必要的入参数据不正确，请检查！");

            ServiceFlow serviceFlow = serviceFlowValid.getServiceFlow();
            ArrayList<ServiceFlow.HandleFile> handleFiles = serviceFlowValid.getHandleFiles();

            if ("java".equals(serviceFlowValid.getServiceFlow().getFlowType())) {
                final boolean[] created = new boolean[1];
                //先生成和编译 processor
                handleFiles.stream().filter(handleFile -> ServiceFlowConstant.FLOW_TYPE_PROCESSOR.equals(handleFile.getUsage())).forEach(handleFile -> {
                    try {
                        if (this.generateClassFile(handleFile)) created[0] = true;
                        else created[0] = false;
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });

                if (!created[0]) {
                    System.out.println("processor 生成失败！");
                    return Result.error("服务流程启动失败！");
                } else {
                    System.out.println("processor 生成成功！");
                }

                handleFiles.stream().filter(handleFile -> ServiceFlowConstant.FLOW_TYPE_ROUTE.equals(handleFile.getUsage())).forEach(handleFile -> {
                    try {
                        if (this.generateClassFile(handleFile)) created[0] = true;
                        else created[0] = false;
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                });
                if (!created[0]) {
                    System.out.println("route文件 生成失败！");
                    return Result.error("服务流程启动失败！");
                } else {
                    System.out.println("route文件 生成成功！");
                }
            } else {
                for (ServiceFlow.HandleFile handleFile : handleFiles) {
                    boolean created = this.createClassFile(handleFile);
                    if (!created) {
                        return Result.error("服务流程启动失败！");
                    }
                }
            }


            this.addRouter(handleFiles);
            SystemCamelContext.startRouter(serviceFlow.getRouteCode());
            List<Route> routes = SystemCamelContext.getContext().getRoutes();

            return Result.success("服务流程启动成功！");
        } catch (Exception e) {
            logger.error(e.getMessage());
            return Result.error("服务流程启动失败！");
        }
    }

    /**
     * 停止路由时，该事件被触发。
     */
    public Result onServiceFlowStop(String msg) {
        try {
            ServiceFlowValid serviceFlowValid = new ServiceFlowValid(msg).invoke();
            if (serviceFlowValid.is()) return Result.error("必要的入参数据不正确，请检查！");

            ServiceFlow serviceFlow = serviceFlowValid.getServiceFlow();
            String routeCode = serviceFlow.getRouteCode();
            SystemCamelContext.getContext().stopRoute(routeCode);
            return Result.success("停止路由成功！");
        } catch (Exception e) {
            return Result.error("停止路由失败！");
        }
    }

    private boolean createClassFile(ServiceFlow.HandleFile handleFile) throws Exception {
        if (handleFile == null) {
            return false;
        }
        SystemCamelContext.putClassMapping(handleFile.getRouteCode(), handleFile.getPackageName(), handleFile.getUsage(), handleFile.getClassName());
        URL resource = SystemCamelContext.getResource(this);
        FileOutputStream outputStream = ClassFileUtil.createFile(resource, handleFile.getPackageName(), handleFile.getClassName(), ClassFileUtil.CLASS_FILE);

        String fileName = handleFile.getClassName() + ClassFileUtil.CLASS_FILE;

        //从中心下载
        String downUrl = fsUrl + "/" + handleFile.getFilePath();
        String sourcePath = ClassFileUtil.downFile(downUrl, resource, handleFile.getPackageName(), handleFile.getClassName(), ClassFileUtil.CLASS_FILE);

        if (sourcePath == null) {
            logger.error("not mongo file, fileName:" + fileName);
            return false;
        }

        return true;
    }

    private boolean generateClassFile(ServiceFlow.HandleFile handleFile) throws Exception {
        if (handleFile == null) {
            System.out.println("=====handleFile is null...");
            return false;
        }
        SystemCamelContext.putClassMapping(handleFile.getRouteCode(), handleFile.getPackageName(), handleFile.getUsage(), handleFile.getClassName());
        URL resource = SystemCamelContext.getResource(this);

        //从中心下载java文件
        String downUrl = fsUrl + "/" + handleFile.getFilePath();
        String sourcePath = ClassFileUtil.downFile(downUrl, resource, handleFile.getPackageName(), handleFile.getClassName(), ClassFileUtil.JAVA_FILE);

        if (sourcePath == null) {
            System.out.println("下载java文件失败，downUrl：" + downUrl);
            return false;
        }

        logger.info(sourcePath);
        boolean succ = CamelCompiler.compile(sourcePath, resource.getPath());
        if (succ) {
            //编译成功后将class文件上传至mongodb；文件名为类名+ routeCode
//            String packagePath = StringUtil.replaceStrAll(handleFile.getPackageName(), ".", "/");
//            String classPath = this.getClass().getResource("/").getPath() + packagePath + "/" + handleFile.getClassName() + ClassFileUtil.CLASS_FILE;
//
//            //上传文件
//            String fileName = handleFile.getClassName() + ClassFileUtil.CLASS_FILE;
//            ClassFileUtil.upload(fsUrl, classPath);
        } else {
            System.out.println("编译失败，sourcePath：" + sourcePath);
        }
        return succ;
    }

    private void deleteClassFile(ServiceFlow.HandleFile handleFile) {
        try {

            String packagePath = StringUtil.replaceStrAll(handleFile.getPackageName(), ".", "/");
            String classPath = this.getClass().getResource("/").getPath() + "/" + packagePath + "/" + handleFile.getClassName() + ".class";
            String javaPath = this.getClass().getResource("/").getPath() + "/" + packagePath + "/" + handleFile.getClassName() + ".java";
            Boolean aBoolean = ClassFileUtil.deleteClassfile(javaPath);
            Boolean bBoolean = ClassFileUtil.deleteClassfile(classPath);
            //删除mongodb中文件
            String delJavaUrl = fsUrl + "/" + handleFile.getFilePath();
            ClassFileUtil.deleteFile(delJavaUrl);

            // 完成
            logger.info("===================" + handleFile.getPackageName() + CoreConstant.DOT + handleFile.getClassName() + ".class 删除过程结束");

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 删除 serviceFlow记录
     *
     * @param routeCode
     */
    private void deleteServiceFlow(String routeCode) {
        try {
            MongoDatabase db = mongoClient.getDatabase(configuration);
            MongoCollection<Document> collection = db.getCollection(serviceFlow);
            collection.deleteMany(Filters.eq("routeCode", routeCode));
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private void addRouter(ArrayList<ServiceFlow.HandleFile> handleFiles) throws Exception {
        ArrayList<ServiceFlow.HandleFile> routerFiles = new ArrayList<>();
        handleFiles.forEach(handleFile -> {
            try {
                if (handleFile.getUsage().equals(ServiceFlowConstant.FLOW_TYPE_ROUTE)){
                    String className = SystemCamelContext.getClassMapping(handleFile.getRouteCode(), handleFile.getClassName(), handleFile.getUsage());
                    SystemCamelContext.getRouterQueue().put(className);
                } else {
                    String className = SystemCamelContext.getClassMapping(handleFile.getRouteCode(), handleFile.getClassName(), handleFile.getUsage());
                    SystemCamelContext.getProcessorQueue().put(className);
                }
            } catch (Exception e) {
                logger.error(e.getMessage());
                e.printStackTrace();
            }
        });

    }

    public String getRouteStatus(String routeId){
        ServiceStatus routeStatus = SystemCamelContext.getContext().getRouteStatus(routeId);
        if (routeStatus!=null){
            return routeStatus.name();
        }else {
            return null;
        }
    }

    private class ServiceFlowValid {
        private boolean myResult;
        private String msg;
        private ServiceFlow serviceFlow;
        private ServiceFlow.HandleFile handleFile;
        private ArrayList<ServiceFlow.HandleFile> handleFiles;

        public ServiceFlowValid(String msg) {
            this.msg = msg;
        }

        public ArrayList<ServiceFlow.HandleFile> getHandleFiles() {
            return handleFiles;
        }

        public ServiceFlow getServiceFlow() {
            return serviceFlow;
        }

        public ServiceFlow.HandleFile getHandleFile() {
            return handleFile;
        }

        public ServiceFlowValid invoke() throws IOException {
            serviceFlow = objectMapper.readValue(msg, ServiceFlow.class);
            handleFiles = serviceFlow.getHandleFiles();
            if (handleFiles == null || handleFiles.size() == 0) {
                logger.error("必要的入参数据不正确，请检查！");
                myResult = true;
                return this;
            }

            handleFile = handleFiles.get(0);

            if (handleFile == null || StringUtil.isEmpty(serviceFlow.getRouteCode()) || StringUtil.isEmpty(handleFile.getPackageName())
                    || StringUtil.isEmpty(handleFile.getPackageName()) || StringUtil.isEmpty(handleFile.getFilePath())) {
                logger.error("必要的入参数据不正确，请检查！");
                myResult = true;
                return this;
            }
            myResult = false;
            return this;
        }

        boolean is() {
            return myResult;
        }
    }
}
